%     Copyright (C) 2023 Jordi Llopis-Lorente. Contact: jorllolo@etsii.upv.es
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>


%%%% When running this code, the simulations of the 15 drugs will be
%%%% launched in the male and female population. For each scenario, a .mat
%%%% file with a set of biomarkers for the individuals in the population is
%%%% generated.


%% Simulation Drugs
%%%%%%%%%%%%%%%
drugs_to_simulate = {'Azimilide', 'Dofetilide', 'Vandetanib', 'Quinidine', 'Disopyramide', 'Sotalol', 'Chlorpromazine', 'Cisapride', 'Clarithromycin', 'Clozapine', 'Droperidol', 'Domperidone', 'Pimozide', 'Risperidone', 'Ondansetron'};
for d = 1:length(drugs_to_simulate)
    drug_to_simulate = drugs_to_simulate{d};
    path_name = 'Drug_scenarios'; % path to the folder containing the excels files with the pharmacological data
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Conditions to simulate
    BCLs = 1000; % basic ccle length (ms)
    celltypes = {'endo'}; % options: 'endo'; 'mid'; 'epi'
    BeatsSaved = 5;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    myCluster=parcluster('local');
    poolobj = gcp('nocreate');
    if isempty(poolobj)
        poolobj = parpool(myCluster.NumWorkers);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    load('factors_population.mat') % It loads the scaling factors of the male and female population.
    % It is a 600x12 matrix. Each row represent an individual of the populations.
    % 1:300 are male model, and 301:600 are female models
    % Each column is contains de scaling factor for a given ion channel: 
    % 1. GNa  2. Gto  3. GCaL  4. GKr  5. GKs  6. GK1  7. GNCX  8. PNaK  9. GKb
    % 10. GpCa   11. GJup  12. calm
    scalars = [parameters];
    %%

    [num,txt,raw] = xlsread([ path_name drug_to_simulate '_maxC_IC50.xlsx']);
    genders = num(:,1);
    FPCs = num(:,2);
    IC50s = num(:,3:end);
    scenarios = txt(2:end,1);

    for cases = 1: size(scenarios,1)
        disp (['Simulating: ' drug_to_simulate ' ' scenarios{cases}])
        D = FPCs (cases);
        IC50 = IC50s(cases,:);
        drug_effect = calculateDrug_effect(D, IC50);
        gender = genders(cases);
        if gender == 1
            disp(['Male population']);
            scaling = scalars(1:300,:);
            sex = 'male';
        elseif gender == 2
            disp(['Female population']);
            scaling = scalars(301:600,:);
            sex = 'female';
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for BCL_ind = 1:length(BCLs)
            BCL = BCLs(BCL_ind);
            for celltypes_ind = 1:length(celltypes)
                celltype = celltypes{celltypes_ind};
                tic
                parfor i=1:size(scaling,1)

                    ind_scaling = scaling(i,:);
                    [time,X,IsJs] = runORdmD_sex(BCL, ind_scaling, drug_effect, gender, BeatsSaved, celltype); 
                    sims(i,:)=X{end,1}(end,:);

                    % Calculate biomarkers
                    [APD40(i),Vpp(i),dVdt(i),Vr(i),Vp(i), RAs(i,:), alternans(i,:)]=APD_beats(time,X,40);
                    [APD30(i)]=APD_beats(time,X,30);
                    [APD50(i)]=APD_beats(time,X,50);
                    [APD90(i)]=APD_beats(time,X,90);
                    Tri90_40(i) = APD90(i) - APD40(i);
                    Tri90_30(i) = APD90(i) - APD30(i);

                    [catd50(i),cai_syst(i),cai_diast(i), RAsCa(i,:), alternansCa(i,:)]=CTD_beats(time,X,50, RAs(i,:));
                    [catd90(i)]=CTD_beats(time,X,90, RAs(i,:));

                    qnet(i) = getQnet(time,IsJs);
                    qnet_apd(i) = qnet(i)./APD90(i);

                    EMw(i) = catd90(i)-APD90(i);

                end

                save([drug_to_simulate '_' scenarios{cases} '_' sex '_population_BCL' num2str(BCL) '_' celltype '.mat'],'sims', 'scaling', 'sex', 'APD90', 'APD50', 'APD40','APD30', ...
                    'Tri90_40', 'Tri90_30', 'Vpp', 'dVdt', 'Vr','Vp','RAs', 'alternans', 'catd90', 'catd50', 'cai_syst','cai_diast', 'RAsCa', 'qnet', 'qnet_apd', 'EMw')
                toc
            end
        end
    end
end
